package com.hive.sample.uihelper;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.lang.ref.SoftReference;
import java.util.LinkedList;
import java.util.Queue;

import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.util.Log;


/**
 * 메모리 캐시 : 4MB (default)<br>
 * 디스크 캐시 : 50MB (default)<br>
 * 동시 가용 스레드 수 : 4 (default)<br>
 * 최대 가용 스레드 수 : 제한없음<br>
 * LRU 캐시 정책 : 사용<br>
 * Reference 정책 : SoftReference
 * 저장 key : url (SHA256)<br>
 * 저장 data : byte[] (original)<br>
 * Optional return type : byte[], bitmap object<br>
 * 
 * @author hife<br>
 * Copyright 2016 GAMEVIL Inc. Com2uS Corp.
 */
public class HiveImageLoader {
	
	private volatile static HiveImageLoader instance = null;
	
	private static final long MEMORY_CACHE_SIZE = 1024 * 1024 * 4;
	private static final long DISK_CACHE_SIZE = 1024 * 1024 * 50;
	private static final int AVAILABLE_THREAD_COUNT = 4;
	
	
	private android.util.LruCache<String, SoftReference<byte[]>> memoryCache = new android.util.LruCache<String, SoftReference<byte[]>>((int)MEMORY_CACHE_SIZE);
//	private Map<String, SoftReference<byte[]>> memoryCache = new Hashtable<String, SoftReference<byte[]>>();
	private String DISK_CACHE_PATH;
	private File diskCacheDir;
    
	private Queue<HiveImageRequestData> taskQueue;
	private HiveImageWorker[] workers;
    
	public static HiveImageLoader getInstance(Context context) {
		if (instance == null) {
			synchronized (HiveImageLoader.class) {
				if (instance == null) {
					instance = new HiveImageLoader(context);
				}
			}
		}
		return instance;
	}
	
	private HiveImageLoader(Context context) {
		Log.v("HiveIAPImageLoader", "[HiveIAPImageLoader] create");
		taskQueue = new LinkedList<HiveImageRequestData>();
		workers = new HiveImageWorker[AVAILABLE_THREAD_COUNT];
		for (int i=0; i< AVAILABLE_THREAD_COUNT; i++) {
			workers[i] = new HiveImageWorker(context);
		}
		
		DISK_CACHE_PATH = context.getCacheDir().getPath() + "/hiveiapcache";
		diskCacheDir = new File(DISK_CACHE_PATH);
		try {
			if(!diskCacheDir.exists()) diskCacheDir.mkdirs();
			Log.d("HiveIAPImageLoader", "[HiveIAPImageLoader] make disk cache dir success: " + DISK_CACHE_PATH);
		} catch(Exception e) {
			Log.w("HiveIAPImageLoader", "[HiveIAPImageLoader] make disk cache dir failed: " + e.toString());
		}
	}
	
	public synchronized void loadImage(String url, final HiveImageLoadingListener listener) {
		Log.d("HiveIAPImageLoader", "[HiveIAPImageLoader] loadImage: " + url);
		if(listener == null) {
			Log.w("HiveIAPImageLoader", "[HiveIAPImageLoader] loadImage: " + url + " listener is null");
			return;
		}
		
		if(TextUtils.isEmpty(url)) {
			Log.w("HiveIAPImageLoader", "[HiveIAPImageLoader] loadImage url is empty");
			listener.onLoadingFailed(url, "loadImage url is empty");
			return;
		}
		
		taskQueue.offer(new HiveImageRequestData(new Handler(Looper.getMainLooper()), url, listener));
		doWork();
	}
	
	synchronized HiveImageRequestData peekTaskQueue() {
		return taskQueue.peek();
	}
	
	synchronized HiveImageRequestData pollTaskQueue() {
		return taskQueue.poll();
	}
	
	byte[] readCache(String url) {
		Log.d("HiveIAPImageLoader", "[HiveIAPImageLoader] readCache: " + url);

		if(TextUtils.isEmpty(url)) return null;
		
		byte[] data = null;
		data = readMemoryCache(url);
		
		if(data != null) return data;
		else data = readDiskCache(url);
		
		return data;
	}

	private byte[] readMemoryCache(String url) {
		Log.d("HiveIAPImageLoader", "[HiveIAPImageLoader] readMemoryCache: " + url);
		
		if(memoryCache.get(url) != null) {
			return memoryCache.get(url).get();
			
		} else return null;
	}
	
	private byte[] readDiskCache(String url) {
		Log.d("HiveIAPImageLoader", "[HiveIAPImageLoader] readDiskCache: " + url);
		
		String hashUrl = makeToSHA256(url);
		
		if(TextUtils.isEmpty(hashUrl)) {
			Log.w("HiveIAPImageLoader", "[HiveIAPImageLoader] readDiskCache invaild hash string: " + hashUrl);
			return null;
		}
		
		File newFile = new File(DISK_CACHE_PATH + "/" + hashUrl);
		if(!newFile.exists()) {
			Log.d("HiveIAPImageLoader", "[HiveIAPImageLoader] not found disk cache file (not exception)");
			return null;
		}
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		FileInputStream fis = null;
		byte buffer[] = new byte[8096];
		int readLen;
		byte[] data = null;
		try {
			fis = new FileInputStream(newFile);
			
			while( (readLen = fis.read(buffer)) > 0) {
				baos.write(buffer, 0, readLen);
			}
			data = baos.toByteArray();
			
			return data;
			
		} catch (Exception e) {
			Log.w("HiveIAPImageLoader", "[HiveIAPImageLoader] read file failed: " + e.toString());
			return null;
			
		} finally {
			if(fis != null) try { fis.close(); } catch (Exception ignore) {}
			if(baos != null) try { baos.close(); } catch (Exception ignore) {}
		}
	}
	
	synchronized void writeCache(String url, byte[] data) {
		Log.d("HiveIAPImageLoader", "[HiveIAPImageLoader] writeCache: " + url);
		
		if(TextUtils.isEmpty(url) || data == null) return;
		
		writeMemoryCache(url, data);
		writeDiskCache(url, data);
	}
	
	private synchronized void writeMemoryCache(String url, byte[] data) {
		Log.d("HiveIAPImageLoader", "[HiveIAPImageLoader] writeMemoryCache: " + url);
		
		//use LruCache
//		long size = 0;
//		for (SoftReference<byte[]> value : memoryCache.values()) {
//			if(value != null && value.get() != null)
//			size += (long)value.get().length;
//		}
//		
//		Log.d("[HiveIAPImageLoader] memory cache size: " + size);
//		
//		if(size > MEMORY_CACHE_SIZE) {
//			memoryCache.clear();
//		}
		
		memoryCache.put(url, new SoftReference<byte[]>(data));
	}
	
	private synchronized void writeDiskCache(String url, byte[] data) {
		Log.d("HiveIAPImageLoader", "[HiveIAPImageLoader] writeDiskCache: " + url);
		
		long size = 0;
		
		if (diskCacheDir.exists()) {
	        File[] cacheFiles = diskCacheDir.listFiles();
	        
			if(cacheFiles != null) {
				for (File cacheFile : cacheFiles) {
					size += cacheFile.length();
				}
			}
			
			Log.i("HiveIAPImageLoader", "[HiveIAPImageLoader] disk cache size: " + size);
			
			if(size > DISK_CACHE_SIZE) {
				Log.i("HiveIAPImageLoader", "[HiveIAPImageLoader] disk cache size overflow. now delete cache files.");
				if(cacheFiles != null) {
					for (File cacheFile : cacheFiles) {
						cacheFile.delete();
					}
				}
			}
	    }
		
		String hashUrl = makeToSHA256(url);
		
		if(TextUtils.isEmpty(hashUrl)) {
			Log.w("HiveIAPImageLoader", "[HiveIAPImageLoader] writeDiskCache invaild hash string: " + hashUrl);
			return;
		}
		
		File newFile = new File(DISK_CACHE_PATH + "/" + hashUrl);
		
		try {
			if(!newFile.getParentFile().exists()) newFile.getParentFile().mkdirs();
			if(!newFile.exists()) newFile.createNewFile();
		} catch (Exception e) {
			Log.w("HiveIAPImageLoader", "[HiveIAPImageLoader] create file failed: " + e.toString());
			return;
		}
		
		FileOutputStream fos = null;
		try {
			fos = new FileOutputStream(newFile);
			fos.write(data);
			fos.flush();
		} catch (Exception e) {
			Log.w("HiveIAPImageLoader", "[HiveIAPImageLoader] write file failed: " + e.toString());
			return;
			
		} finally {
			if(fos != null) try { fos.close(); } catch (Exception ignore) {}
		}
	}
	
	private void doWork() {
		for (HiveImageWorker worker : workers) {
			worker.work();
		}
	}
	
	private String makeToSHA256(String str){
		String SHA = ""; 
		try{
			java.security.MessageDigest sh = java.security.MessageDigest.getInstance("SHA-256"); 
			sh.update(str.getBytes()); 
			byte byteData[] = sh.digest();
			StringBuffer sb = new StringBuffer(); 
			for(int i = 0 ; i < byteData.length ; i++){
				sb.append(Integer.toString((byteData[i]&0xff) + 0x100, 16).substring(1));
			}
			SHA = sb.toString();
			
		}catch(Exception e){
			Log.w("HiveIAPImageLoader", "[HiveIAPImageLoader] makeToSHA256 exception: " + e.toString());
			SHA = null; 
		}
		return SHA;
	}
}
