/*
 * Copyright (C) 2007 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hive.sample.uihelper;

import android.widget.*;
import android.content.*;
import android.os.Handler;
import android.os.Message;
import android.text.method.*;
import android.text.Editable;
import android.util.AttributeSet;




/**
 * This is a TextView that is Editable and by default scrollable, like EditText without a cursor.
 * 
 * @author ryuvsken
 */
public class LogTextView extends TextView {
	
	private static final int MOVETO_TOP = 1;
	private static final int MOVETO_UP = 2;
	private static final int MOVETO_DOWN = 3;
	private static final int MOVETO_BOTTOM = 4;
	
	
	public int defaultScrollLine = 5;
	
	
	// 안드로이드의 TextView 는 UI 이벤트에서 scrollTo() 를 하면 메시지 내용이 사라지기 때문에 핸들 이벤트로 돌려야 한다.
    private Handler scrollHandler = new Handler() {
    	
        //private static final int REFRESH_INTERVAL = 1000;

        public void handleMessage(Message msg) {
        	
        	switch(msg.what) {
        	
        		case MOVETO_TOP: {
        			
        			scrollTo(0, 0);
        			break;
        		}
        		
        		case MOVETO_UP: {
        			
        	        // int l_nLineCount = getLineCount();
        	    	int l_nPixelsPerLine = getLineHeight();
        	    	//int temp = getScrollY();
        	    	int scrollLine = msg.arg1;
        	    	int l_nCurrentHeight = getScrollY() - (scrollLine * l_nPixelsPerLine);
        	        int l_nViewHeight = getHeight(); 

        	        // The difference between view height and total text height 
        	        int l_nDifference = l_nCurrentHeight;
        	        if (l_nDifference > l_nViewHeight) {
        	            scrollTo(0,l_nDifference - l_nViewHeight);
        	        }
        	        else {
        	        	scrollTo(0, l_nDifference);
        	        }
        	        break;
        		}
        		
        		case MOVETO_DOWN: {
        			
        	        // int l_nLineCount = getLineCount();
        	    	int l_nPixelsPerLine = getLineHeight();
        	    	//int temp = getScrollY();
        	    	int scrollLine = msg.arg1;
        	    	int l_nCurrentHeight = getScrollY() + (scrollLine * l_nPixelsPerLine);
        	        int l_nViewHeight = getHeight(); 

        	        // The difference between view height and total text height 
        	        int l_nDifference = l_nCurrentHeight;
        	        if (l_nDifference > l_nViewHeight) {
        	            scrollTo(0,l_nDifference - l_nViewHeight);
        	        }
        	        else {
        	        	scrollTo(0, l_nDifference);
        	        }
        	        break;
        		}
        		
        		case MOVETO_BOTTOM: {
        			
        	        int l_nLineCount = getLineCount(); 
        	        int l_nViewHeight = getHeight(); 
        	        int l_nPixelsPerLine = getLineHeight(); 

        	        // The difference between view height and total text height 
        	        int l_nDifference = ( l_nLineCount * l_nPixelsPerLine );
        	        if (l_nDifference > l_nViewHeight) {
        	            scrollTo(0,l_nDifference - l_nViewHeight);
        	        }
        	        break;
        		}
        	}
        	
        	requestLayout();
            
            // sendEmptyMessageDelayed(0, REFRESH_INTERVAL);
        }
    };
    
    
    public LogTextView(Context context) {
        this(context, null);
    }

    public LogTextView(Context context, AttributeSet attrs) {
    	
        this(context, attrs, android.R.attr.textViewStyle);
    }

    public LogTextView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
    }

    
    @Override
    protected MovementMethod getDefaultMovementMethod() {
    	
        return ScrollingMovementMethod.getInstance();
    }
    
    
    /**
     * 기본 스크롤 라인 갯수 반환
     */
    public int getDefaultScrollLine() {
    	
    	return this.defaultScrollLine;
    }
    
    
	/**
	 * 기본 스크롤 라인 갯수 설정
	 * @param defaultScrollLine	기본 스크롤 라인 갯수
	 */
    public void setDefaultScrollLine(int defaultScrollLine) {
    	
    	this.defaultScrollLine = defaultScrollLine;
    }
    

    @Override
    public Editable getText() {
    	
        return (Editable) super.getText();
    }

    
    @Override
    public void setText(CharSequence text, BufferType type) {
    	
        super.setText(text, BufferType.EDITABLE);
        
        int l_nLineCount = getLineCount(); 
        int l_nViewHeight = getHeight(); 
        int l_nPixelsPerLine = getLineHeight(); 

        // The difference between view height and total text height 
        int l_nDifference = ( l_nLineCount * l_nPixelsPerLine );
        if (l_nDifference > l_nViewHeight) {
            scrollTo(0,l_nDifference - l_nViewHeight);
        }
    }
    
    
    /**
     * 로그뷰에 로그 추가
     * 
     * @param logString	로그 메시지
     */
    public void append(String logString) {
    	
    	if(!logString.endsWith("\n"))
    		logString += "\n";
    	
        super.append(logString);
        
        int l_nLineCount = getLineCount(); 
        int l_nViewHeight = getHeight(); 
        int l_nPixelsPerLine = getLineHeight(); 

      //The difference between view height and total text height 
        int l_nDifference = ( l_nLineCount * l_nPixelsPerLine );
        if (l_nDifference > l_nViewHeight) {
            scrollTo(0,l_nDifference - l_nViewHeight);
        }
    }
    
    
    /**
     * 로그뷰의 내용을 모두 지운다.
     */
    public void clear() {
        super.setText("");
        scrollTo(0,0);
    }
    
    
    /**
     * 로그뷰의 커서를 최상위로 이동
     */
    public void moveTop() {
    	
    	//this.scrollTo(0, 0);
    	//scrollHandler.sendEmptyMessage(1);
    	Message msg = Message.obtain();
    	msg.what = MOVETO_TOP;
    	scrollHandler.sendMessage(msg);
    }
    
    
    /**
     * 로그뷰의 커서를 주어진 값만큼 상위로 이동
     * 
     * @param line	상위로 이동한 라인 수
     */
    public void moveUp(int line) {
    	
    	Message msg = Message.obtain();
    	msg.what = MOVETO_UP;
    	
    	if(line <= 0) {
    		line = defaultScrollLine;
    	}
    	msg.arg1 = line;
    	scrollHandler.sendMessage(msg);
    }
    
    
    /**
     * 로그뷰의 커서를 주어진 값만큼 하위로 이동
     * 
     * @param line	하위로 이동한 라인 수
     */
    public void moveDown(int line) {
    	
    	Message msg = Message.obtain();
    	msg.what = MOVETO_DOWN;
    	if(line <= 0) {
    		line = defaultScrollLine;
    	}
    	msg.arg1 = line;
    	scrollHandler.sendMessage(msg);
    }
    
    
    /**
     * 로그뷰의 커서를 최하위로 이동
     */
    public void moveBottom() {
    	
    	Message msg = Message.obtain();
    	msg.what = MOVETO_BOTTOM;
    	scrollHandler.sendMessage(msg);

    }
}



